import { jwtDecode, gql } from '../App'

const actionPending = name => ({ type: 'PROMISE', status: 'PENDING', name })
const actionResolved = (name, payload) => ({ type: 'PROMISE', status: 'RESOLVED', name, payload })
const actionRejected = (name, error) => ({ type: 'PROMISE', status: 'REJECTED', name, error })
export const actionPromise = (name, promise) =>
    async dispatch => {
        dispatch(actionPending(name))
        try {
            let payload = await promise
            dispatch(actionResolved(name, payload))
            return payload
        }
        catch (error) {
            dispatch(actionRejected(name, error))
        }
    }

export const actionAuthLogin = token => ({ type: 'AUTH_LOGIN', token })
export const actionAuthLogout = () => ({ type: 'AUTH_LOGOUT' })
export const actionLogin = (login, password) =>
    actionPromise('login', gql(`
  query log($login:String!, $password:String!) {
      login(login: $login, password: $password)
    }`, { login, password }))

export const actionFullLogin = (login = 'tst', password = '123') =>
    async dispatch => {
        let token = await dispatch(actionLogin(login, password))
        if (token) {
            await dispatch(actionAuthLogin(token))
            await dispatch(actionGetUserData())
            dispatch(actionGetUserPlaylists())
            dispatch(actionGetUserTracks())
        }
    }

export const actionRegister = (login, password) =>
    actionPromise('registration', gql(`
        mutation register($login:String!, $password:String!) {
            createUser(login: $login, password: $password) {
                login, _id
            }
        }
        `, { login, password })
    )

export const actionFullRegister = (login = 'tst', password = '123') =>
    async dispatch => {
        await dispatch(actionRegister(login, password))
        await dispatch(actionFullLogin(login, password))
    }

export const actionGetUserData = () => {
    let _id = jwtDecode(localStorage.authToken).sub.id
    return (
        actionPromise('userData', gql(`
            query($userId: String!) {
                UserFindOne(query: $userId){
                    login, _id, avatar {_id, url, originalFileName}
                }
            }
        `, { userId: JSON.stringify([{ _id }]) }))
    )
}

export const actionGetUserPlaylists = () => {
    let _id = jwtDecode(localStorage.authToken).sub.id
    return(
        actionPromise('userPlaylists', gql(`
            query getPlaylistByOwnerId($ownerId:String!) {
                PlaylistFind(query: $ownerId) {
                    _id, name
                }
            }
        `, { ownerId: JSON.stringify([{ ___owner: _id }]) } ))
    )
}

export const actionGetPlaylistById = (_id='5fe35e5ce926687ee86b0a4f') =>
    actionPromise('playlistTracks', gql(`
        query playlistById($playlistId: String!) {
            PlaylistFind(query: $playlistId) {
                _id, 
                name,
                tracks {
                    url, originalFileName,
                    id3{ title, artist, album },
                }
            }
        }
    `, { playlistId: JSON.stringify([{ _id }]) }))

export const actionGetUserTracks = () => {
    let _id = jwtDecode(localStorage.authToken).sub.id
    //let _id = '5fe35e1ce926687ee86b0a3f' //newUserId
    return(
        actionPromise('userTracks', gql(`
            query getUserTracks($ownerId: String!) {
                TrackFind(query: $ownerId) {
                    _id, originalFileName, url,
                    id3 { title, artist, album }
                }
            }
        `, { ownerId: JSON.stringify([{ ___owner: _id }]) } ))
    )
}

export const actionAddPlaylist = playlistName => 
    async dispatch => {
        await dispatch(actionPromise('addPlaylist', gql(`
            mutation addPlaylist ($playlistName: String!){
                PlaylistUpsert(playlist: {name: $playlistName}) {
                    _id, name
                }
            }
        `, {playlistName: playlistName})))
        dispatch(actionGetUserPlaylists())
    }